@@ -58,6 +58,8 @@ class UnitNo:
     def set(self, *args):
         """See :class:`UnitNo` constructor for details of arguments."""
         if len(args) == 1:
+            if args[0] == "nan":
+                args[0] = None
             if args[0]:
                 if isinstance(args[0], list) or isinstance(args[0], tuple):
                     return self.set(*args[0])
@@ -181,19 +183,22 @@ class ObsNo:
 
     def set(self, *args):
         """See :class:`ObsNo` constructor for details of arguments."""
-        if len(args) == 1 and args[0]:
-            if isinstance(args[0], list) or isinstance(args[0], tuple):
-                return self.set(*args[0])
-            for pattern in PATTERNS["obs_no"]:
-                match = re.match(pattern, args[0])
-                if match:
-                    self.plan = match.group(1)
-                    self.seq = int(match.group(2))
-                    return
-            raise ValueError(
-                "no identifier found in {}, "
-                "check docs for accepted formats".format(args[0])
-            )
+        if len(args) == 1:
+            if args[0] == "nan":
+                args[0] = None
+            if args[0]:
+                if isinstance(args[0], list) or isinstance(args[0], tuple):
+                    return self.set(*args[0])
+                for pattern in PATTERNS["obs_no"]:
+                    match = re.match(pattern, args[0])
+                    if match:
+                        self.plan = match.group(1)
+                        self.seq = int(match.group(2))
+                        return
+                raise ValueError(
+                    "no identifier found in {}, "
+                    "check docs for accepted formats".format(args[0])
+                )
         elif len(args) == 2:
             if isinstance(args[0], str):
                 self.plan = args[0]
@@ -254,6 +259,8 @@ class Well:
             "200135".
         title (str): available attributes including name, e.g.
             "7025-3985 / WRG038 / WESTERN LAGOON".
+        obs_no (ObsNo): obs number
+        unit_no (UnitNo): unit number
 
     """
 
@@ -293,6 +300,14 @@ class Well:
         """
         self.unit_no.set(*args)
 
+    @property
+    def unit_hyphen(self):
+        return self.unit_no.hyphen
+
+    @property
+    def unit_long(self):
+        return self.unit_no.long
+
     def __eq__(self, other):
         if hasattr(other, "dh_no"):
             return self.dh_no == other.dh_no
@@ -326,7 +341,12 @@ class Well:
         return " / ".join(names)
 
     def __repr__(self):
-        return "<sa_gwdata.Well({}) {}>".format(self.dh_no, self.title)
+        if self.obs_no:
+            return f"'{str(self.obs_no)}'"
+        elif self.unit_hyphen:
+            return f"'{str(self.unit_hyphen)}'"
+        else:
+            return str(self.dh_no)
 
     def to_scalar_dict(self):
         """Convert Well to a dictionary containing scalar values.
@@ -353,12 +373,13 @@ class Well:
         """Return title containing only characters which are allowed in
         Windows path names."""
         r = str(self)
-        r = r.replace(" /", ";")[1:-1]
         for char in ["\\", "/", "?", ":", "*", '"', "<", ">", "|"]:
             r = r.replace(char, "")
-        if remove_prefix:
-            parts = r.split(")")
-            r = " ".join(parts[1:])[1:]
+
+        # This keyword argument now has no function.
+        # if remove_prefix:
+        #     parts = r.split(")")
+        #     r = " ".join(parts[1:])[1:]
         return r
 
 
@@ -384,7 +405,7 @@ class Wells(collections.abc.MutableSequence):
         if wells is None:
             wells = []
         self.wells = wells
-        self._update_attribute_names()
+        self._refresh()
 
     def __repr__(self):
         return repr(self.wells)
@@ -393,22 +414,31 @@ class Wells(collections.abc.MutableSequence):
         return len(self.wells)
 
     def __getitem__(self, ix):
-        return self.wells[ix]
+        if isinstance(ix, int):
+            if ix < len(self):
+                return self.wells[ix]
+        key = ix
+        if not key in self._map:
+            for id_type, value in parse_well_ids_plaintext(str(key)):
+                if value in self._map:
+                    key = value
+                    break
+        return self._map[key]
 
     def __delitem__(self, ix):
         del self.wells[ix]
-        self._update_attribute_names()
+        self._refresh()
 
     def __setitem__(self, ix, value):
         self.wells[ix] = value
 
     def insert(self, ix, value):
         self.wells.insert(ix, value)
-        self._update_attribute_names()
+        self._refresh()
 
     def append(self, value):
         self.wells.append(value)
-        self._update_attribute_names()
+        self._refresh()
 
     def count(self, item):
         return self.wells.count(item)
@@ -430,11 +460,14 @@ class Wells(collections.abc.MutableSequence):
                 "Wells object does not have an attribute named '{}'".format(name)
             )
 
-    def _update_attribute_names(self):
+    def _refresh(self):
         if len(self):
             self._attributes = list(self[0].to_scalar_dict().keys())
         else:
             self._attributes = []
+        self._map = {w.dh_no: w for w in self}
+        self._map.update({w.obs_no.id: w for w in self if w.obs_no.id})
+        self._map.update({w.unit_no.hyphen: w for w in self if w.unit_no.hyphen})
 
     def __dir__(self):
         return sorted(
